//----------------------------------------------------------------------------
// 
//	file: net_window.cc
// 
//	function:  lattice - like net display of coorrdinates 
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Fri Aug  7 03:52:00 CDT 1998
// 
//	$Revision: 1.2 $ $Date: 2000/02/13 10:29:43 $
//	$Log: net_window.cc,v $
//	Revision 1.2  2000/02/13 10:29:43  yschoe
//	*** empty log message ***
//
//	Revision 1.1  1998/08/11 16:57:14  yschoe
//	display mesh-like network coordinates
//
//
//----------------------------------------------------------------------------
#include <window.h>
#include <string.h>
#include <iostream.h>
#include <stream.h>
#include "net_window.h"
#include "ppmout.h"

//----------------------------------------------------------------------------
//	net_window() 
//	  - Constructors
//----------------------------------------------------------------------------
net_window::net_window
	(window &p, int w, int h, int x, int y, int bw, 
	int color_range, int color_mode, 
	int cols, int rows, // matrix size
        REAL min_x, REAL max_x, REAL min_y, REAL max_y, // grid size
        coord **cog_matrix) 
: plot_window(p,w,h,x,y,bw,color_range,color_mode,0,0) {

  init(cols,rows,min_x,max_x,min_y,max_y,cog_matrix,NULL);

}

//----------------------------------------------------------------------------
net_window::net_window
	(window &p, int w, int h, int x, int y, int bw, 
	int color_range, int color_mode, 
	int cols, int rows, // matrix size
        REAL min_x, REAL max_x, REAL min_y, REAL max_y, // grid size
        coord **cog_matrix, coord **ideal) 
: plot_window(p,w,h,x,y,bw,color_range,color_mode,0,0) {

  init(cols,rows,min_x,max_x,min_y,max_y,cog_matrix,ideal);

}

//----------------------------------------------------------------------------
//	~net_window() 
//	  - Destructor
//----------------------------------------------------------------------------
net_window::~net_window(){

  // XFreePixmap(display,pix); done elsewhere?
  delete [] color_map;

  // delete ideal[][]
  if (ideal_alloced == NET_WIN_TRUE) {
  	for (int i=0; i<cols; ++i) delete [] ideal[i];
  	delete [] ideal;
  }

}

//----------------------------------------------------------------------------
//	draw_interior() 
//	  - main drawing routine : use redraw() when you need to invoke this
//----------------------------------------------------------------------------
void net_window::draw_interior() {

  // draw ideal grid

  set_color(cyan);
  DrawGrid(ideal);

  set_color(black);
  DrawGrid(cog_matrix);
  
}


//----------------------------------------------------------------------------
//	DrawGrid() 
//	  - draw grid of the coordinate matrix
//----------------------------------------------------------------------------
void net_window::DrawGrid(coord **grid) {

  for (int x=0; x<cols; ++x) 
     for (int y=0; y<rows-1; ++y) 
	DrawLine(grid[x][y].x,grid[x][y].y,grid[x][y+1].x,grid[x][y+1].y);	

  for (int x=0; x<cols-1; ++x) 
     for (int y=0; y<rows; ++y) 
	DrawLine(grid[x][y].x,grid[x][y].y,grid[x+1][y].x,grid[x+1][y].y);	
    
}

//----------------------------------------------------------------------------
//	init() 
//	  - initialize common stuff
//----------------------------------------------------------------------------
void net_window::init(int cols, int rows, 
	REAL min_x, REAL max_x, REAL min_y, REAL max_y,
	coord **cog_matrix, coord **ideal_grid) {

  net_window::color_range = color_range;

  cerr << "matrix_window::init color_range = " << color_range << endl;

  plot_window::init(0);

  // set ranges
  net_window::cols = cols;
  net_window::rows = rows;
  net_window::cog_matrix = cog_matrix;
  x_max = max_x;
  y_max = max_y;
  x_min = min_x;
  y_min = min_y;

  // Actually allocate colors
  color_map = new int[color_range]; // allocate table for cmap
  alloc_colors();

  // allocate or assign ideal grid
  if (ideal_grid==NULL) {

  	// allocate ideal grid
  	ideal = new coord* [cols];
  	for (int i=0; i<cols; ++i) ideal[i] = new coord [rows];

  	// set up ideal grid
  	ideal_alloced = NET_WIN_TRUE;
  	for (int x=0; x<cols; ++x) {
     	   for (int y=0; y<cols; ++y) {
		// since for n columns there are n-1 equal sections
		// in between the range, we use cols-1 and rows-1
     		ideal[x][y].x = ((x_max-x_min)/(REAL)(cols-1))*(x-x_min);
     		ideal[x][y].y = ((y_max-y_min)/(REAL)(rows-1))*(y-y_min);
     	   }
  	}

  } else { // if ideal_grid is non-NULL

 	// link up the ideal grid
  	ideal_alloced = NET_WIN_FALSE;
	net_window::ideal = ideal_grid;
	// must find x_max, y_max, x_min, y_min.
	x_max = -99999999.9; y_max = -99999999.9;
	x_min =  99999999.9; y_min = 99999999.9;
	for (int x=0; x<cols; ++x) {
	   for (int y=0; y<cols; ++y) {
		x_max = MAX(ideal[x][y].x,x_max);
		y_max = MAX(ideal[x][y].y,y_max);
		x_min = MIN(ideal[x][y].x,x_min);
		y_min = MIN(ideal[x][y].y,y_min);
	   }
     	}   

  } // end else

}
