//----------------------------------------------------------------------------
// 
//	file: netlayer.h
// 
//	function:  defines a single neural network layer 
//		let's assume the default is 2D, and 1D should be
//		implemented as a column vector (vector[0][n]).
//		- Worry about all other simulation details as
//		  parameter update schedule, radius reduction etc,
//		  to the simulation class.
//
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Mon Jul 20 04:20:35 CDT 1998
// 
//	$Revision: 1.16.1.14 $ $Date: 2001/11/30 20:24:15 $
//	$Log: netlayer.h,v $
//	Revision 1.16.1.14  2001/11/30 20:24:15  yschoe
//	pglissom v1.0 release
//
//	Revision 1.16.1.13  2000/02/22 03:54:53  yschoe
//	*** empty log message ***
//
//	Revision 1.16.1.12  2000/02/08 11:25:21  yschoe
//	*** empty log message ***
//
//	Revision 1.16.1.11  1999/02/09 07:42:25  yschoe
//	1. circular aff weights (i.e. RFs)
//	2. instar/outstar option for exc weight :get_lat_excw
//	3. or_pref calculation fixed and util routines for or_pref added
//	4. dumping aff/lat weights routine added.
//
//	Revision 1.16.1.10  1999/01/10 04:16:45  yschoe
//	1. added or_pref calculation and print routines
//	2. added center of gravity calculation
//
//	Revision 1.16.1.9  1998/11/29 08:53:58  yschoe
//	fixed get_wgt routines -- base substraction was incorrect
//	, and added display option for instar or outstar normlaization in the
//	get inh weight function. The end result seems comparable.
//
//	Revision 1.16.1.8  1998/11/27 06:03:38  yschoe
//	1. check for killed weights.
//	2. added kill_overlapping_weights()
//	3. normalization by sum of wgt (not sqrt of sum of sqrs)
//		-- the inline function was not being overridden in
//		   derived classes!!
//
//	Revision 1.16.1.7  1998/11/17 12:40:42  yschoe
//	cleaned up weight normalization and added normalize_instar_weights().
//
//	Revision 1.16.1.6  1998/11/06 20:03:13  yschoe
//	fixed huge bug : normalizing outstar now
//
//	Revision 1.16.1.5  1998/10/26 03:52:39  yschoe
//	see netlayer.cc 1.16.1.5
//
//	Revision 1.16.1.4  1998/10/16 10:08:04  yschoe
//	added randomness.
//
//	Revision 1.16.1.3  1998/10/05 19:48:46  yschoe
//	1. added get_inh|exc|affw_matrix() functions
//	2. added setup_weight() functions
//	3. separated calc_affresp and calc_latresp from calc_netinput
//	4. set_matrix() for gaussian weight setup
//
//	Revision 1.16.1.2  1998/09/26 10:31:39  yschoe
//	1. made normalization of affweights to "by-sum"
//	2. activate() and calc_netinput() no longer have default arg.
//
//	Revision 1.16.1.1  1998/09/10 07:03:06  yschoe
//	removed a single '-' from the act_func() function
//
//	Revision 1.16  1998/09/05 05:04:38  yschoe
//	changed version string to '#define'
//
//	Revision 1.15  1998/09/05 04:59:26  yschoe
//	added version string
//
//	Revision 1.14  1998/08/26 02:41:33  yschoe
//	Made inline functions non-virtual.
//
//	Revision 1.13  1998/08/22 08:18:50  yschoe
//	added net_aff/exc/inh_input and changed
//	interface to activate(). calc_netinput(), learn(), and normalize_weights()
//	were made outstar.
//
//	Revision 1.12  1998/08/14 17:44:29  yschoe
//	1. fixed bug in resize_lat_exc_rad().
//	2. added kill_lat_weights.
//
//	Revision 1.11  1998/08/14 06:50:29  yschoe
//	1. fixed little cosmetic bug in destructor
//	2. added resize_lat_exc_rad()
//
//	Revision 1.10  1998/08/11 17:01:09  yschoe
//	1. Modularized norm_sum() and norm_factor() for easier
//	extension of weight normalization.
//	2. Added reset_matrix() and reset_output()
//
//	Revision 1.9  1998/08/02 11:05:30  yschoe
//	1. calc_netinput() separated from activate().
//	2. fixed set_parameter(), get_parameter() and added different interfaces.
//
//	Revision 1.8  1998/07/30 00:18:53  yschoe
//	1. fixed destructor, init_weights(),
//	   normalize_weights() to use precalculated ranges.
//	2. fixed serious bug in activate(), weighted sum loop.
//	3. added learn(), delta(), and get_parameter().
//
//	Revision 1.7  1998/07/28 06:44:22  yschoe
//	1. init_weights() can now take a range [low,high)
//	2. activate() was added
//	3. set_parameter() was added --> will be removed later (maybe)
//	.
//
//	Revision 1.6  1998/07/27 00:00:00  yschoe
//	added normalize_weights()
//	     update_blackboard(int cur_inp)
//	     calc_rf_ranges(...)
//
//	Revision 1.5  1998/07/26 02:11:55  yschoe
//	init_weights(), act_func() added.
//	sig_low, sig_high added and added to blackboard.
//	must add normalization code.
//
//	Revision 1.4  1998/07/24 11:22:08  yschoe
//	save() load() and list() blackboard
//	routines added
//
//	Revision 1.3  1998/07/24 08:22:24  yschoe
//	see comment in netlayer.cc
//
//	Revision 1.2  1998/07/23 08:45:32  yschoe
//	alloc/dealloc of weights, setting up variables done. 
//	Todo: adding everything to blackboard, initializing weights, 
//	and all the real action stuff.
//
//	Revision 1.1  1998/07/23 01:07:05  yschoe
//	jul22 stable version (except netlayer)
//
//
//----------------------------------------------------------------------------
#ifndef __NETLAYER_H
#define __NETLAYER_H

#include <iostream.h>
#include <fstream.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "real.h"
#include "blackboard.h"
#include "coord.h"
#include "parser.h"
#include "geninput.h"
#include "matrix.h"
#include "ppmio.h"

#define netlayer_version "netlayer $Revision: 1.16.1.14 $"

#define DEATH_FLAG (1.0E-10)
#define KILLED	  0 

extern "C" {
extern void srand48(long int);
}

enum NETWGT_TYPE { NETWGT_AFF, NETWGT_INH, NETWGT_EXC, NETWGT_CIRCULAR_AFF };

// structure for Receptive Field (RF) range : to speed up calc
typedef struct {
  	int cols, rows;
	int lowx, highx; // x range of RF 
	int lowy, highy;	 // y range of RF (not used in 1D map)
} NET_RF_Range;

// structure for weights
typedef struct {
	REAL	***aff_ws;	// afferent wgts (from one or more areas)
  	REAL	**lat_exc_w;	// lateral excitatory weights
	REAL	**lat_inh_w;	// lateral inhibitory weights
} NET_Weights;

//----------------------------------------------------------------------------
//      CLASS: netlayer
//		defines a single neural network layer and it's operations
//----------------------------------------------------------------------------
class netlayer {

  REAL MIN_REAL;// small real number

protected:
  char  name[BB_NAME_LEN];		// name of the layer

public:

  int	rows, cols;	// layer dimension (size)
  REAL	sig_low, sig_high; // thresh and ceiling for the sigmoid
  REAL  sig_diff;  	// sig_high - sig_low
  int	padded_flag;	// default is 1 00Nov5

public:  // private simulation parameters
  // General network parameters
  int	max_inp_layers; // maximum number of input layers
  int	num_inp_layers;	// number of areas feeding input to this layer

  int	*aff_rads;	// afferent RF radius : as many as num_inp_layers
  int	lat_exc_rad;	// lateral excitatory radius
  int	lat_inh_rad;	// lateral inhibitory radius

  // Flags
  int	recursive_lateral_flag;	// exc and inh connection to self?
  int	normalization_mode_flag; // instar or outstar normalization?

  REAL	threshold;	// base threshold for firing

  REAL	*aff_contris;	// afferent contribution : as many as num_inp_layers
  REAL  lat_exc_contri;	// lateral excitatory contribution
  REAL	lat_inh_contri;	// lateral inhibitory contribution

  REAL	*aff_noises;	// afferent noise ratio : as many as num_inp_layers
  REAL	lat_exc_noise;	// lateral excitatory noise
  REAL	lat_inh_noise;	// lateral inhibitory noise

  REAL	intrinsic_noise;// noise in neuron activity

  REAL	**wgt_sum; // weight sum for normalization
  REAL  **or_pref; // orientation preference (degrees)
  REAL  **or_selectivity; // orientation preference (degrees)

public:

  REAL	*alpha_affs;	// afferent learning rate : as many as num_inp_layers
  REAL	alpha_lat_exc;	// lateral excitatory learning rate
  REAL  alpha_lat_inh;	// lateral inhibitory learning rate

  // These are 2D matrices, for map layer [x][y].
  REAL		**net_input;	// net input 
  REAL		**net_aff_input;// net input from other layer(s) : num_inp_layer
  REAL		**net_exc_input;// excitatory net input from same layer
  REAL		**net_inh_input;// inhibitory net input from same layer
  NET_Weights	**wgt;		// weights 

  NET_RF_Range  ***rf_ranges;	// RF range in the feeding areas : num_inp_layer
				// Calculate from RF radius
  NET_RF_Range  **lat_exc_range;// lat exc conn range 
  NET_RF_Range  **lat_inh_range;// lat inh conn range 

  // Where the inputs come from (= other layers)
  netlayer 	**feedings;	// input from (point to the layer(s))
				// pointer to where the output goes is not 
				// needed -- the receiving end must take care
				// of it. As many as num_inp_layers.
  coord 	***cogs; 	// Center of gravity of afferents


  // Other administrative members
  blackboard *net_bb;	// blackboard for image dump/load

  virtual REAL act_func(REAL x) { //activation function 

  if (x <= sig_low)
        return 0.0;
  else if (x >= sig_high)
        return 1.0;
  else
        return (x-sig_low)/sig_diff;

  }


public:

  REAL		**output;	// output (after act_func) of this layer
  REAL		**prev_output;	// output from previous iteration

	 netlayer(char *name, int cols, int rows, int exc_rad=0, int inh_rad=0,
                 int max_inp_layers=0, netlayer *feeding= (netlayer *)NULL,
                 int inp_cols=0, int inp_rows=0, int  aff_rad=0, 
		int padded_flag=1);
	~netlayer();

  // setup and add link to input layer
  virtual void  add_inp_layer(netlayer *feeding, 
		 int inp_cols, int inp_rows,
		 int aff_rad); // add input layer to this layer

  virtual void  save(char *fileprefix);
  virtual void  load(char *fileprefix);
  virtual void  list();		// list blackboard

  virtual void	init_weights(REAL low, REAL high);
  virtual void  normalize_weights(); // use divide by sum for aff and lat
  virtual void  normalize_instar_weights();// use divide by sum for aff and lat
  virtual void	activate(int calc_aff); // if calc_aff==1, calc net aff input
  virtual void	learn();
  virtual void	learn_v();
  inline REAL delta_lat(int col, int row, int inp_col, int inp_row,
			REAL alpha);
  inline REAL delta_aff(int col, int row, int cur_inp, int inp_col, 
			int inp_row);

  // parameter setting routines
  virtual void  set_parameter(char *command);
  virtual void  set_parameter(parser *sim_parser);
  virtual void  set_parameter(char *param_name, REAL value);
  virtual void  set_parameter(char *param_name, int col, REAL value);
  virtual void  set_parameter(char *param_name, int col, int row, REAL value);
  virtual REAL  get_parameter(char *param_name);
  virtual REAL  get_parameter(char *param_name, int col);
  virtual REAL  get_parameter(char *param_name, int col, int row);
  virtual void  reset_output();

  // topology changing routines, and other bookkeeping stuff
  void 	resize_lat_exc_rad(int new_rad);
  int   call_resize_lat_exc_rad(int argc, char *argv[]);
  void 	resize_lat_inh_rad(int new_rad);
  int   call_resize_lat_inh_rad(int argc, char *argv[]);
  void  bookkeep_parameters();

  void  kill_lat_weights(REAL threshold);
  int   call_kill_lat_weights(int argc, char *argv[]);
  void  update_sig(REAL low, REAL high); // update sig_high,low,and diff
  int   call_update_sig(int argc, char *argv[]);

  // weight pruning 
  void kill_wgt(int x, int y, NETWGT_TYPE type, int cur_inp=0);
  void kill_aff_wgt_range(int sx, int sy, int ex, int ey, 
		int cur_inp);

  int  call_kill_aff_wgt_range(int argc, char *argv[]); 
  void kill_lat_exc_wgt_range(int sx, int sy, int ex, int ey);
  int  call_kill_lat_exc_wgt_range(int argc, char *argv[]); 

  void kill_instar_lat_wgt(int x, int y, NETWGT_TYPE type);
  void kill_instar_lat_exc_wgt_range(int sx, int sy, int ex, int ey);
  int  call_kill_instar_lat_exc_wgt_range(int argc, char *argv[]); 

  // returns pointer and information to weights of individual neurons
  matrix_coord get_affw_matrix(int i, int j, int cur_inp=0);
  matrix_coord get_inhw_matrix(int i, int j);
  matrix_coord get_excw_matrix(int i, int j);

  // setup weights
  void setup_weight(NETWGT_TYPE type, int radius, REAL randomness,
		int cur_input=0);
  void setup_weight(NETWGT_TYPE type, REAL sigma, REAL randomness,
		int cur_input=0);
  void setup_weight(NETWGT_TYPE type, REAL sigmax, REAL sigmay, REAL angle , 
		REAL randomness, int cur_input=0);

  // kill inh weights that overlap with exc weights
  void kill_overlapping_inh_w();

  // return affw and lateral weight in a matrix
  void get_affw(REAL **dest_matrix, int col, int row, int cur_inp = 0);
  void get_lat_excw(REAL **dest_matrix, int col, int row, int is_instar = 0);
  void get_lat_inhw(REAL **dest_matrix, int col, int row, int is_instar = 0);
  void get_lat_combinedw(REAL **dest_matrix, int col, int row);

  // return only the area covered by actual weights : do not return
  // surrounding areas
  void get_affw_only(REAL **dest_matrix, int col,int row,int cur_inp=0);
  void get_lat_excw_only(REAL **dest_matrix,int col,int row, 
		int x_size, int y_size,
		int is_instar = 0);
  void get_lat_inhw_only(REAL **dest_matrix,int col,int row, 
		int x_size, int y_size,
		int is_instar = 0);

  // dump weights
  void dump_wgts(int col, int row, int cur_inp = 0, 
	NETWGT_TYPE choice=NETWGT_AFF, FILE *fp=stdout);
  int  call_dump_wgts(int argc, char *argv[]);
  int  call_dump_wgts_range(int argc, char *argv[]); 
  int  call_dump_wgts_range_select(int argc, char *argv[]);

  // dump weights : only area covered by connections
  void dump_wgts_only(int col, int row, int cur_inp = 0, 
	NETWGT_TYPE choice=NETWGT_AFF, FILE *fp=stdout);

  // dump weight sum
  void dump_wgt_sums(int col, int row, int cur_inp = 0);
  int  call_dump_wgt_sums(int argc, char *argv[]);
  int  call_dump_wgt_sums_range(int argc, char *argv[]); 

  // make RF circular
  void circular_affw(REAL **dest_matrix,int cols, int rows, 
		int cx, int cy, int cur_inp=0);

  // analisis routines : orientation preference/selectivity, etc.
  void calc_or_pref(int cur_inp, REAL sigmax, REAL sigmay);
  REAL calc_single_or_pref(int cur_inp, int i, int j, 
		REAL sigmax, REAL sigmay);
  void calc_single_or_pref_vecsum(int cur_inp, int i, int j, 
		REAL sigmax, REAL sigmay, REAL *pref, REAL *selec);
  void print_or_pref();
  void save_or_pref(int cur_inp, char *filename);
  void save_or_selectivity(int cur_inp, char *filename);
  int  call_save_or_pref(int argc, char *argv[]);
  int  call_save_or_selectivity(int argc, char *argv[]);

  // calculate center of gravity
  void calc_cog(int cur_inp_layer);
  void save_cog(int cur_inp_layer, char *filename);

  // clear output[][]
  void clear_activity() { reset_matrix(output,cols,rows); }

  // dump or difference histogram for lateral weights
  void dump_wgt_or_histo(int col, int row, REAL thresh,
                NETWGT_TYPE choice=NETWGT_EXC, FILE *fp=stdout, 
		int omit_radius=0);
  int call_dump_wgt_or_histo(int argc, char *argv[]);
  void dump_wgt_or_histo_weighted(int col, int row, REAL thresh,
                NETWGT_TYPE choice=NETWGT_EXC, FILE *fp=stdout);
  int call_dump_wgt_or_histo_weighted(int argc, char *argv[]);
  void dump_wgt_or_stat(int col, int row, REAL thresh,
                NETWGT_TYPE choice=NETWGT_EXC, FILE *fp=stdout, 
		int omit_radius=0, int angle_low = -90, int angle_high = 90);
  int call_dump_wgt_or_stat(int argc, char *argv[]);
  int call_dump_wgt_or_stat_limited_angle(int argc, char *argv[]);

  // plot matrix
  void plot_matrix(char *param_name);
  int call_plot_matrix(int argc, char *argv[]);
  int call_plot_stat(int, char **);

  // dump matrix
  void dump_matrix(char *param_name,int t);
  int call_dump_matrix(int argc, char *argv[]);

protected:
	
  // memory allocation/deallocation routines
  virtual void	alloc_first(int cols, int rows, int max_inp_layers, 
				int exc_rad, int inh_rad);
  virtual REAL         **alloc_matrix(int cols, int rows); 
  virtual NET_RF_Range **alloc_matrix(int c,int r,NET_RF_Range **rf_range); 
  virtual NET_Weights  **alloc_matrix(int cols, int rows,NET_Weights **wgts); 
  virtual int	       **alloc_matrix(int cols, int rows,int **mat); 

  virtual void delete_matrix(int cols, int rows, REAL **rp);
  virtual void delete_matrix(int cols, int rows, NET_RF_Range **rfp);
  virtual void delete_matrix(int cols, int rows, NET_Weights **wgts);
  virtual void delete_matrix(int cols, int rows, int **mat);

  virtual void init_blackboard(char *name);
  virtual void update_blackboard(int cur_inp);
  virtual void add_lat_connections();
  virtual void calc_rf_ranges(int cur_inp, int cols, int rows, int inp_cols, 
				int inp_rows, int aff_rad);
  virtual void calc_netinput(int calc_aff);
  virtual void calc_affresp();
  virtual void calc_affresp_padded();
  virtual void calc_latresp();
  virtual void calc_latresp_optimized();

  // 00jun6
  virtual void calc_latresp_exc(REAL **activity);
  virtual void calc_latresp_inh(REAL **activity);

  void reset_matrix(int **mat, int cols, int rows);
  void reset_matrix(REAL **mat, int cols, int rows);
  void apply_actfunc();
  void unuse_actfunc();
  //calculate sum of aff and lat contributions
  void calc_contrib_sum(int calc_aff);
  void calc_contrib_sum_mult(int calc_aff);

  // gaussian matrix setup
  void  set_matrix(REAL **matrix, int cols, int rows, 
		int cx, int cy, int radius, REAL randomness);
  void  set_matrix(REAL **matrix, int cols, int rows, 
		int cx, int cy, REAL sigma, REAL randomness);
  void  set_matrix(REAL **matrix, int cols, int rows, 
		int cx, int cy, REAL sigmax, REAL sigmay,
                REAL angle, REAL randomness);
  void  set_matrix(REAL **mat, int cols, int rows, REAL randomness);

private: 

  // for or_pref calculation
  REAL  **ref_pattern;

  // These inline functions could not be overridden in
  // derived classes, which is a big problem for now. 98dec18 
  inline REAL norm_factor(REAL sum) { // aff normalization factor
	//return sqrt(sum); 
	return sum;
  }; 
  inline REAL norm_sum(REAL sum) { // Used when summing afferent wgts
	//return sum*sum; 		   //  for normalization.
	return sum;
  }; 

  // bounds check (x1,y1, x2,y2) 
  inline int bounds_check(int x1, int y1, int x2, int y2) {
	return ( x1>=0 && x1<cols && y1>=0 && y1<rows && 
		 x2>=0 && x2<cols && y2>=0 && y2<rows);
  }

public:

  // testing if this works with simenv class
  int addition (int argc, char *argv[]);

  int is_norm_by_sum() { // check if normalization is done by sum of wgts
	if (norm_sum(2.0)>2.0) return 0; else return 1;
  }

  REAL calc_norm_sum(REAL sum) {
	return norm_sum(sum);
  }

  REAL sum(REAL** mat) {
	REAL sum = 0.0;

	for (int i=0; i<cols; ++i) {
	   for (int j=0; j<rows; ++j) {
	      sum += mat[i][j];
	   }
	}

	return sum;
	
  }

  REAL avg(REAL** mat) {

	REAL s = sum(mat);
	return (s/((REAL)(cols*rows)));
	
  }

  void minmax(REAL** mat, REAL* min, REAL* max) {

	*min = 99999;
	*max = -99999;
	
	for (int i=0; i<cols; ++i) {
	   for (int j=0; j<rows; ++j) {
	      *min = (*min>mat[i][j])?mat[i][j]:*min;
	      *max = (*max<mat[i][j])?mat[i][j]:*max;
	   }
	}
  }

  void nonzero_minmax(REAL** mat, REAL* min, REAL* max) {

	*min = 99999;
	*max = -99999;
	
	for (int i=0; i<cols; ++i) {
	   for (int j=0; j<rows; ++j) {
	      *min = (*min>mat[i][j] && mat[i][j]>0.001)?mat[i][j]:*min;
	      *max = (*max<mat[i][j])?mat[i][j]:*max;
	   }
	}
  }
  	

};

#endif
