// TODO: must replace cog calculation with the one in netlayer class
//----------------------------------------------------------------------------
// 
//	file: lissom.cc
// 
//	function: defines and implements LISSOM 
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Wed Aug  5 18:04:55 CDT 1998
// 
//	$Revision: 1.18 $ $Date: 1999/02/09 07:56:44 $
//	Log: lissom.cc,v -- see rlog
//
//----------------------------------------------------------------------------
#define lissom_version "lissom.cc $Revision: 1.18 $"

#include <iostream.h>
#include <nnview/netlayer.h>
#include <nnview/simulog.h>
#include <nnview/real.h>
#include <nnview/coord.h>
#include <nnview/geninput.h>

#ifndef LissomCUI
#include <window.h>
#include <nnview/plot_window.h>
#include <nnview/matrix_window.h>
#include <nnview/tool_window.h>
#include <nnview/net_window.h>
#include <nnview/disputil_window.h>
#include <nnview/gdb_visualize.h>
#endif


#ifndef LissomCUI
main_window     *mw;
plot_window     *pw ;
plot_window     *pw2 ;
plot_window     *pw3 ;
matrix_window   *matw;
matrix_window   *inpw;
matrix_window   *matw2;
tool_window     *tw;
net_window	*nw;
callback_button *runit;
callback_button *saveit;
#endif

int             steps;
int 		t=0;
simulog		*netlog;

netlayer	*input;
netlayer	*map;

coord		**cog;
coord		**ideal;

geninput	*input_generator;

// Network topology and some other parameters
int N = 20;
int M = 20;
int GRAN = 10;
REAL randomness=1.0;
REAL sigma=1.0;
int iters = 101;
int disp_rate = 1;
int iters_per_run = 10;
int tsettle = 9;

//----------------------------------------------------------------------------
//	chop() 
//	  - chop last entry in a string
//----------------------------------------------------------------------------
void chop(char *str) {

  str[strlen(str)-1] = '\0';

}

//----------------------------------------------------------------------------
//      setup() 
//        - setup network for simulation
//----------------------------------------------------------------------------
void setup(int argc, char *argv[], char *configfile) {

  netlog = new simulog("lissom.cc","lissom.log");

  netlog->append("command line %s %s ...",argv[0],argv[1]);
  netlog->append("code version: %s",lissom_version);
  netlog->append("code version: %s",netlayer_version);
  netlog->append("N = %d (map NxN)",N);
  netlog->append("M = %d (input MxM)",M);
  netlog->append("randomness = %f",randomness);

  // input layer
  cout << "Initializing input layer.\n";
  input = new netlayer("input",M,M,0,0,0);
  
  // map layer
  cout << "Initializing map layer.\n";
  map   = new netlayer("map",N,N,1,N,1,input,M,M,M);

  // add local simulation parameters to the 
  // blackboard 
  map->net_bb->add("GRAN",&GRAN);
  map->net_bb->add("randomness",&randomness);
  map->net_bb->add("sigma",&sigma);
  map->net_bb->add("iters",&iters);
  map->net_bb->add("disp_rate",&disp_rate);
  map->net_bb->add("iters_per_run",&iters_per_run);
  map->net_bb->add("tsettle",&tsettle);

  // set parameter values
  //   read from file.
  FILE *conffp = fopen(configfile,"r");
  char buff[100];

  // read parameters
  int count = 0;
  while (!feof(conffp)) {
    	fgets(buff,100,conffp);
	if (feof(conffp)) { break; }
        map->set_parameter(buff);
	chop(buff);
	netlog->append("%s",buff);
	count ++;
  }
  cout << "lissom:setup() " << count << " commands processed\n";
  fclose(conffp);

  // init weights
  map->init_weights(0.0,0.0);
  map->setup_weight(NETWGT_AFF,(REAL)3.0,randomness);
  map->setup_weight(NETWGT_INH,(REAL)5.0,randomness);
  map->setup_weight(NETWGT_EXC,(REAL)2.0,randomness);
  map->normalize_weights();

  // set up ideal grid
  // ideal = NULL;
  ideal = new coord* [N];
  for (int i=0; i<N; ++i) 
	ideal[i] = new coord [N];

  for (int i=0; i<map->cols; ++i) {
    for (int j=0; j<map->rows; ++j) {
       ideal[i][j].x = (REAL) i*M/(REAL)N;
       ideal[i][j].y = (REAL) j*M/(REAL)N;
    }
  }

  // set up COG (center of gravity) map
  cog = new coord* [N];
  for (int i=0; i<N; ++i) 
	cog[i] = new coord [N];

}

//----------------------------------------------------------------------------
//      run() 
//        - bplayer : run a certain number of backprop learning
//----------------------------------------------------------------------------
void run () {

  char str[100];
  int i;

  // for the number of given steps
  for (i=t; i<=t+iters_per_run; ++i) {

    cout << "t = " << i << endl;

    // set input and normalize
    REAL sum=0.0;
    input_generator->clear_input();
    input_generator->add_input((int)(drand48()*M),
			(int)(drand48()*M),
			(REAL)sigma, (REAL)sigma, (REAL)(drand48()*360.0));
#   ifndef LissomCUI
    inpw->redraw();
    XFlush(display);
#   endif

    // clear map activity
    map->reset_output();

    // activate and settle
    map->activate(1); // calculate aff input (just once)
    for (int k=0; k<tsettle; ++k) {

	map->activate(0); // don't calc aff while settling

	// display
        if (i%disp_rate==0) {
#	    	ifndef LissomCUI
		// calculate c.o.g.
		map->calc_cog(cog,0);
		matw->redraw();
    		nw->redraw();
#	    	else
		if (k==0) {
			cout << "iter " << i << endl;
		}
#	    	endif
	}	
#       ifndef LissomCUI
	set_color(white);
	XFillRectangle(display,mw->Win,gc_copy,
		0,mw->height-20,mw->width,20);	
	set_color(black);
	sprintf(str,"settle=%d",k);
	mw->DrawString(0,mw->height-10,str);
	sprintf(str,"iter=%d",i);
	mw->DrawString(0,mw->height,str);
	mw->redraw();
	XFlush(display);
#       endif
    }

    // learn weights
    //if (i!=0) { // somehow it doesn't work the first time!!
       map->learn();
       map->normalize_weights();
    //}

    // update sigmoid
    if (i<1300) {
	map->update_sig((map->sig_high - map->sig_low)
			/ 2.0 / (steps/20.0),map->sig_high);
      	// map->sig_low += (map->sig_high-map->sig_low)/2.0/(steps/20.0);
     	// cout << "new sig_low = " << map->sig_low << endl; 
      	//map->sig_high -= (map->sig_high-map->sig_low)/4.0/(steps/10.0);
    }

    // reduce learing rate
    /*
    if (i%600==0 && i!=0) { 
	if (map->alpha_affs[0]>0.001) {
      	REAL new_alpha = map->alpha_affs[0];
      	map->alpha_affs[0] = new_alpha * 0.1;
      	map->alpha_lat_exc = map->alpha_lat_exc * 0.1 ;
      	map->alpha_lat_inh = map->alpha_lat_inh * 0.1 ;
       	cout << "new alpha = " << new_alpha << endl;     
	}
    }
    */
	
    // reduce radius
    if (i==300) {
	cout << "lissom.cc:run():  reducing radius.\n";
	netlog->append("iter %d: lissom.cc:run():  reducing radius to %d\n",
		i, 2);
	map->resize_lat_exc_rad(2);
    }
    if (i==600) {
	cout << "lissom.cc:run():  reducing radius.\n";
	netlog->append("iter %d: lissom.cc:run():  reducing radius to %d\n",
		i, 1);
	map->resize_lat_exc_rad(1);
    }

    // kill connections
    if (i%1000==0 && i!=0 && i<3000) {
	cout << "lissom.cc:run():  killing lat inh weights.";
	netlog->append("iter %d: lissom.cc:run():  killing lat inh wgts.",i);
	map->kill_lat_weights(0.00001);
    }

    // log every 1000 iterations
    if (i%disp_rate==0)
  	netlog->append("iter %d reached.",i);

  }

  // set start of next run
  t = i;

}


//----------------------------------------------------------------------------
//	save() 
//	  - save blackboard
//----------------------------------------------------------------------------
void save() {

  cout << "lissom.cc: saveit() : saving to lissom.sav\n";
  map->net_bb->save("lissom.sav");
  //dispmat(map->output,36,36,5);

}
//----------------------------------------------------------------------------
//	main() 
//	  - the MAIN
//----------------------------------------------------------------------------
int main(int argc, char *argv[]) {

  cout << "This program is obsolete : use lissom2.cc \n";
  exit(0);

  steps = 20000;
  t 	= 0;

  // 1. check args
  if (argc < 2) {
	cout << "usage: " << argv[0] 
	     << " <configfilename> [<network imagefile>] [<start t>]\n";
 	exit(0);
  } 

  // 2.1 setup network
  setup(argc,argv,argv[1]);

  // 2.2 if network imagefilename given, load the network image
  if (argc >= 3) {
	cout << "lissom.cc : main() : loading image " << argv[2] << "\n";
	map->net_bb->load(argv[2]);
  // read parameters as well
  FILE *conffp = fopen(argv[1],"r");
  char buff[100];
  int count = 0;
  while (!feof(conffp)) {
    	fgets(buff,100,conffp);
	if (feof(conffp)) { break; }
        map->set_parameter(buff);
	chop(buff);
	netlog->append("%s",buff);
	count ++;
  }
  cout << "lissom:setup() " << count << " commands processed\n";
  fclose(conffp);
  }

  if (argc >= 4) {
	cout << "lissom.cc : main() : setting t = " << argv[3] << "\n";
	t = atoi(argv[3]);
  }

  // 3. setup input generator
  input_generator = new geninput(input->output,M,M,INP_ORI_BAR,INP_NORM_SQRT);
  input_generator->clear_input();

# ifndef LissomCUI

  //
  // 4.1. GUI Interface
  //

  // setup the windows
  mw = new main_window(argv[0],N*GRAN*2,N*GRAN*2+40);
  
  matw = new matrix_window(*mw,N*GRAN,N*GRAN,0,0,0,60,COLOR,N,N,map->output);  

  inpw = new matrix_window(*mw,M*GRAN,M*GRAN,0,N*GRAN+10,0,60,
		COLOR,M,M,input->output);  

  runit = new callback_button(*mw,"Run",run,
                mw->width/4,20,mw->width/2,mw->height-20);

  saveit = new callback_button(*mw,"Save",save,
		mw->width/4,20,mw->width*1/4,mw->height-20);
 
  nw = new net_window(*mw,N*GRAN,N*GRAN,N*GRAN,0,0,GRAN,
		COLOR,N,N,0.0,1.0,0.0,1.0,cog,ideal);

  new quit_button(*mw,mw->width/4,20,mw->width*3/4,mw->height-20);

  matw->redraw();
  nw->redraw();

  // start main loop
  mw->main_loop();

# else

  //
  // 4.2. Text interface
  //
  run();
  map->save("tlissom.sav");

# endif

  delete netlog;

}

/*
  XPoint p1,p2;
  void BPress_1_CB(XButtonEvent ev) {
    p1.x = ev.x; p1.y = ev.y;
    p2 = p1;
  }
  void BRelease_CB(XButtonEvent ) {
    if (fabs(p2.x-p1.x) + fabs(p2.y-p1.y) > 20) {
      main_window *mw= newman(x_org(p1.x),x_org(p2.x),y_org(p2.y),y_org(p1.y));
      mw->RealizeChildren();
    }
  }

*/
