#!/usr/bin/perl
require "getopts.pl";
&Getopts('Gi:g:n:as');

$usage = <<EOF;
usage: compare.pl [-G] [-s] -n <ncols> [-i "<idx1>:<idx2>:..."] [-g <groupstring>] <outfile1> <outfile2> ....

	calculate auto and cross correlation of files in outfile1, outfile2,
	and create a gif (-G option) or PS plot with error bars.

	-G : gif output

	-a : autocorrel only

	-n : number of columns

	-s : use std instead of 95% confidence interval

	"<idx1>:<idx2>: ..." : x index values for outfile1, outfile2, etc.
			Default: 0:1:2:3:...

	<groupstring> : specify groups for auto and cross correlation 
			calculation
			Default: "1,2,3" 4 5 6 7 8 ...
		
	<outfile1> ...	: the slissom2-2 outpufiles to process
EOF

#--------------------
# process arguments
#--------------------
if ($#ARGV<0) {
	print $usage;
	exit;
}

if (!opt_n) {
	print $usage;
	exit
}

if (!$opt_i) { # default to 0:1:2:...
	$opt_i = "0";
	for ($i=1; $i<$opt_n; ++$i) {
		$opt_i .= ":$i";
	}
}
	
if (!$opt_g) { # default to "1,2,3" 4 5 6 7 ...
	$opt_g = '"1,2,3"';
	for ($i=4; $i<=$opt_n; ++$i) {
		$opt_g .= " ".$i;
	}
}

$stdflag = $opt_s?"-s":"";

@idx = split(/:/,$opt_i);

$min = 999999;
$max = -999999;
foreach $range (@idx) {
	if ($range>$max) {
		$max = $range;
	}
	if ($range<$min) {
		$min = $range;
	}
}

$buffer = ($max-$min)/10;
print "** $buffer\n";

$min = $min-$buffer;
$max = $max+$buffer;

$count = 0;
$str = "";
foreach $file (@ARGV) {

	$execstr="plotcorrel2 -nodisp $opt_n $file $opt_g";
	system($execstr);

	# post condition is that $file.autocorrel2.dat and 
	# $file.crosscorrel2.dat exist

	$bname = `basename $file .out`;
	chop $bname;
	
	$str .= $idx[$count++]." ";
	$execstr = "confint.pl $stdflag $bname.autocorrel2.dat | awk \'{print \$4,\$5,\$6}\'";
	$str .= `$execstr`;
	chop $str;
	$str .= " ";
	$execstr = "confint.pl $stdflag $bname.crosscorrel2.dat | awk \'{print \$4,\$5,\$6}'";
	$str .= `$execstr`;
	print $str."\n";

}

open(FP,"> summarydat.conf");
print FP $str;
close(FP);

$imgmode = $opt_G?"-g":"";

$ls = "set pointsize 3; ";
$ls .= "set linestyle 1 lt 2 lw 2 pt 4; ";
$ls .= "set linestyle 2 lt 2 lw 2 pt 5; ";
$ls .= "set linestyle 3 lt 2 lw 2 pt 6; ";
$ls .= "set linestyle 4 lt 2 lw 2 pt 7";

if ($opt_a) {

$columnstr .= "1:2:3:4";
`mplotdat $imgmode -e "set term postscript eps \\"Helvetica\\" 30; set ylabel \\"Avg Correl (CI 95\\%)\\"; set xrange [$min:$max]; set xlabel \\"Orientation Jitter (deg)\\";" -t " " -l "auto" -c $columnstr -w "errorbar" summarydat.conf`

} else {

$columnstr .= "1:2:3:4,1:5:6:7";
`mplotdat $imgmode -e "set term postscript eps \\"Helvetica\\" 30; set ylabel \\"Avg Correl (CI 95\\%)\\"; set xrange [$min:$max]; set xlabel \\"Orientation Jitter (deg)\\";" -t " " -l "auto:cross" -c $columnstr -w "errorbar" summarydat.conf summarydat.conf`

}
