#!/usr/bin/perl

$usage = <<EOF ;
usage: ormapview [-r] <configfile> <iteration> <zoom> [<configfile-with-inputspec>]
	-r : flip map1 and map2
EOF

require "getopts.pl";
&Getopts('r');

$reversep = ($opt_r)?1:0;
$rev_flag = ($reversep)?"-r":"";

if ($#ARGV < 1) {
  print $usage;
  exit;
}

if ($#ARGV == 1) {
	$zoom = 3;
} else {
	$zoom = $ARGV[2];
}

$configfile = $ARGV[0];
$iteration  = $ARGV[1];
$inputconffile = ($#ARGV==3)?$ARGV[3]:"";

#----------------------------------------
# Gather info from config file
#----------------------------------------

open(CONF,$configfile) or die "ormapview: cannot open config file\n";

$mua_count = 0;
$mua_count2 = 0;
$cog_count = 0;
while (<CONF>) {

  if (/^set\ N\ =/) {
	($junk,$junk,$junk,$N,$junk) = split(/[\s]+/,$_,5);
  }

  if (/^set\ M\ =/) {
	($junk,$junk,$junk,$M,$junk) = split(/[\s]+/,$_,5);
  }

  if (/^set\ N2\ =/) {
	($junk,$junk,$junk,$N2,$junk) = split(/[\s]+/,$_,5);
  }

  if (/^set\ RF_RAD\ =/) {
	($junk,$junk,$junk,$RF_RAD,$junk) = split(/[\s]+/,$_,5);
  }

  if (/call_add_mua_spec_circle[\s]/) {
	chop;
	($junk,$junk,$junk,$junk,$junk,$xc,$yc,$or) = split(/[\s]+/,$_); 
	$mua1_list[$mua_count++] = "$xc,$yc";
  }

  if (/call_add_mua_spec_circle2/) {
	chop;
	($junk,$junk,$junk,$junk,$junk,$xc,$yc,$or) = split(/[\s]+/,$_); 
	$mua2_list[$mua_count2++] = "$xc,$yc";
  }

  if (/call_add_spec_ori_gaus/) {
	chop;
	($junk,$junk,$junk,$junk,$junk,$cogx,$cogy,$junk,$junk,$or) = split(/[\s]+/,$_); 
	$input_list[$cog_count] = "$or,$cogx,$cogy";
	$mua1_list[$cog_count] .= ",$input_list[$cog_count]";
	print STDERR $mua1_list[$cog_count]."\n";
	$mua2_list[$cog_count] .= ",$input_list[$cog_count]";
	print STDERR $mua2_list[$cog_count]."\n";
	$cog_count ++;
  }

}

close CONF;

#----------------------------------------
# Generate file list
#----------------------------------------

($confprefix,$junk) = split(/\./,$configfile,2);

$n = ($reversep)?$N2:$N;
$n2 = ($reversep)?$N:$N2;

$orfile = sprintf("$confprefix.%06d.map%d-0",$iteration,($reversep)?2:1);
$orfile2 = sprintf("$confprefix.%06d.map%d-0",$iteration,($reversep)?1:2);

#----------------------------------------
# Generate ppm files
#----------------------------------------
system ("hsvmat $n $n $orfile.or.ppm < $orfile.or");
system ("hsvmat $n2 $n2 $orfile2.or.ppm < $orfile2.or");

#----------------------------------------
# Run printcoord3.tcl command and get input coordinate specs
#---------------------------------------- 
if ( -f "$orfile.or.selec.ppm") {
$cmdstr = "printcoord2.tcl $n $zoom $orfile.or.selec.ppm $orfile.or $n2 $orfile2.or.selec.ppm $orfile2.or $orfile.cog $orfile2.cog $M $RF_RAD $inputconffile|"; 
open(COORDS,$cmdstr);
} else {
$cmdstr = "printcoord2.tcl $n $zoom $orfile.or.ppm $orfile.or $n2 $orfile2.or.ppm $orfile2.or $orfile.cog $orfile2.cog $M $RF_RAD $inputconffile|"; 

print "** running : $cmdstr \n";
}

# Get last line's coordinate dump
while (<COORDS>) {
	chop;
	$specstr = $_;
	$specstr =~ s/^://g;
}
close COORDS;

#----------------------------------------
# Run mktestspec to generate the input spec portion of the simul config file
#----------------------------------------
print STDERR " ** ormapview.pl : Saving input spec to -- inp** \n";
$unpadded_retina_size = $M - 2*$RF_RAD;
$resume_iteration = $iteration+1;
$mua_radius = $RF_RAD*2.0/3.0 * $n/$unpadded_retina_size;
system ("mktestspec $rev_flag $unpadded_retina_size $n $RF_RAD $resume_iteration 3.5 1.5 $mua_radius \"$specstr\" $n2 > inp");

